import torch
import torch.nn as nn
import torch.nn.functional as F

import numpy as np

import torchvision
import torchvision.datasets as datasets

import os

from rendering import matrix_save_image, matrix_rgb_save

dataset_root = "../../torch_dataset"

def fmnist(train):
	fnist=datasets.FashionMNIST(dataset_root, download=True, train=train)
	fnist_x = (fnist.data/255).view(-1,1,28,28)
	fnist_y = F.one_hot(fnist.targets).float()
	return (fnist_x,fnist_y)

def mnist(train):
	mnist=datasets.MNIST(dataset_root, download=True, train=train)
	mnist_x = (mnist.data/255).view(-1,1,28,28)
	mnist_y = F.one_hot(mnist.targets).float()
	return (mnist_x,mnist_y)

from sklearn.metrics import confusion_matrix,precision_score,recall_score,accuracy_score,f1_score

# confusion matrix
def mnist_perf(model,device,set_name,set_x,set_y,batch_size,rgb=[1,0,0]):
	classes = [0,1,2,3,4,5,6,7,8,9]
	y_re = torch.tensor([])
	y_nn = torch.tensor([])
	for batch in range(0,set_x.shape[0],batch_size):
		x = set_x[batch:batch+batch_size].to(device)
		y = set_y[batch:batch+batch_size]
		y_re = torch.hstack((y_re,torch.argmax(      y , dim=1)))
		y_nn = torch.hstack((y_nn,torch.argmax(model(x), dim=1).cpu()))
	# 
	cm        = confusion_matrix(y_re,y_nn,labels=classes,)
	# 
	# FP = np.sum(cm.sum(axis=0) - np.diag(cm) )
	# FN = np.sum(cm.sum(axis=1) - np.diag(cm) )
	# TP = np.sum(np.diag(cm)                  )
	# TN = np.sum(cm.sum() - (FP + FN + TP)    )
	# precision = TP/(TP+FP)
	# recall    = TP/(TP+FN)
	# accuracy  = (TP+TN)/(TP+TN+FP+FN)
	# F1        = 2*(precision*recall)/(precision+recall)
	# 
	precision = precision_score (y_re,y_nn,labels=classes, average='macro', zero_division='warn')
	recall    = recall_score    (y_re,y_nn,labels=classes, average='macro', zero_division='warn')
	accuracy  = accuracy_score  (y_re,y_nn)
	F1        = f1_score        (y_re,y_nn,labels=classes, average='macro', zero_division='warn')
	# print(FP+FN+TP+TN)
	# print(f"{precision} {recall} {accuracy} {F1}")
	# print(f"precision={precision}")
	# print(f"recall={recall}")
	# print(f"accuracy={accuracy}")
	# print(f"F1={F1}")
	cm2 = np.nan_to_num(cm  /cm.sum(axis=0)) # normalize with respect to true labels
	# cm[ground_truth][model_found]
	os.makedirs(f"out/",exist_ok=True)
	# matrix_save_image(cm2, f'out/{model.identifier}-confusion-{set_name}.png')
	matrix_rgb_save(cm2,rgb, f'out/{model.identifier}-confusion-{set_name}.png')
	return precision,recall,accuracy,F1,cm

# def model(x):
# 	model.identifier="DEBUG"
# 	return torch.tensor([1,0,0,0,0,0,0,0,0,0]).repeat(x.shape[0],1).to(device)

# # fmnist_T_x,fmnist_T_y = fmnist(False)
# device="cuda"

# precision,recall,accuracy,F1,cm = mnist_perf(model,device,"debugging",fmnist_T_x,fmnist_T_y,1024)







# # confusion matrix
# def mnist_perf(model,device,set_name,set_x,set_y,batch_size):
# 	classes = [0,1,2,3,4,5,6,7,8,9]
# 	# cm = np.zeros((10,10))
# 	# precision = []
# 	# recall    = []
# 	# accuracy  = []
# 	# F1        = []
# 	y_re = torch.tensor([])
# 	y_nn = torch.tensor([])
# 	for batch in range(0,set_x.shape[0],batch_size):
# 		x = set_x[batch:batch+batch_size].to(device)
# 		y = set_y[batch:batch+batch_size]
# 		y_re = torch.hstack((y_re,torch.argmax(      y , dim=1)))
# 		y_nn = torch.hstack((y_nn,torch.argmax(model(x), dim=1).cpu()))
# 		# print(torch.argmax(      y , dim=1))
# 		# print(torch.argmax(model(x), dim=1))
# 		# correct_rate += torch.sum(y_re-y_nn==0).item() / n
# 	cm        = confusion_matrix(y_re,y_nn,labels=classes,)
# 	precision = precision_score (y_re,y_nn,labels=classes, average='micro', zero_division='warn')
# 	recall    = recall_score    (y_re,y_nn,labels=classes, average='micro', zero_division='warn')
# 	accuracy  = accuracy_score  (y_re,y_nn)
# 	F1        = f1_score        (y_re,y_nn,labels=classes, average='micro', zero_division='warn')
# 		# cm = cm + confusion_matrix(y_re,y_nn,labels=classes,)
# 		# precision .append(precision_score(y_re,y_nn,labels=classes, average='micro', zero_division='warn'))
# 		# recall    .append(recall_score   (y_re,y_nn,labels=classes, average='micro', zero_division='warn'))
# 		# accuracy  .append(accuracy_score (y_re,y_nn))
# 		# F1        .append(f1_score       (y_re,y_nn,labels=classes, average='micro', zero_division='warn'))
# 	# precision = np.mean(precision)
# 	# recall    = np.mean(recall   )
# 	# accuracy  = np.mean(accuracy )
# 	# F1        = np.mean(F1       )
# 	cm2 = np.nan_to_num(cm  /cm.sum(axis=0)) # normalize with respect to true labels
# 	# cm[ground_truth][model_found]
# 	# print(cm)
# 	# precision = []
# 	# recall = []
# 	# accuracy = []
# 	# F1 = []
# 	# for c in classes:
# 	# 	TP = cm[c][c]
# 	# 	FP = sum(cm[:][c]) - cm[c][c]
# 	# 	TN = sum(cm[:][:]) - sum(cm[c][:])
# 	# 	FN = sum(cm[c][:]) - sum(cm[c][c])
# 	# 	print(f"class{c} TP={TP} FP={FP} TN={TN} FN={FN}")
# 	# 	precision.append( TP/(TP+FP))
# 	# 	recall   .append( TP/(TP+FN))
# 	# 	accuracy .append( (TP+TN)/(TP+TN+FP+FN))
# 	# 	F1       .append( 2*(precision[c]*recall[c])/(precision[c]+recall[c]))
# 	# print(f"{precision[0]} {recall[0]} {accuracy[0]} {F1[0]}")
# 	os.makedirs(f"out/",exist_ok=True)
# 	# np.save(f'out/{model.identifier}-confusion-{set_name}-raw.npy',np.array(cm))
# 	# np.save(f'out/{model.identifier}-confusion-{set_name}.npy',np.array(cm2))
# 	matrix_save_image(cm2, f'out/{model.identifier}-confusion-{set_name}.png')
# 	# print(f"correct rate on {name} set = {correct_rate/N}")
# 	# correct_rate=np.sum(np.diag(cm)/np.sum(cm))
# 	return precision,recall,accuracy,F1,cm
# 	# return cm

